@@ -1,5 +1,6 @@
 PKG <- "@PKG@"
-exloc <- file.path("../../../../library", PKG, "R-ex")
+RLIB <- "@RLIB@"
+exloc <- file.path(RLIB, PKG, "R-ex")
 if(!file.exists(exloc)) stop("no examples found")
 list.of.files <- list.files(exloc, ".*\\.R")
 file <- paste(PKG, "-Ex.R", sep="")
@@ -14,7 +15,7 @@ cat(file=file, append=T,
 cat(file=file, append=T, 'options(contrasts = c(unordered = "contr.treatment", ordered =  "contr.poly"))\n')
 cat(file=file, append=T, 'options(pager="console")\n')
 if(PKG != "base")
-    cat(file=file, append=T, paste('library(', PKG, ')', "\n"), sep="")
+    cat(file=file, append=T, paste('library(', PKG, ', lib.loc="', RLIB, '")', "\n", sep=""))
 for(f in list.of.files) {
     cat(file=file, append=T,
         'rm(list = ls(all=TRUE)); .Random.seed <- c(0,rep(7654,3))\n')
