@@ -14,14 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "Graphics.h"
-/* Declarations (for devX11.c) which are not only there,
- * ------------ but also, e.g., in  system.c  [event loop]
- */
-int isX11DeviceActive(void);
-void SaveX11DeviceAsGif(char *filename);
-int X11DeviceDriver(DevDesc*, char*, double, double, double);
+int GADeviceDriver(DevDesc*, char*, double, double, double);
 
