@@ -48,7 +48,7 @@
 /*	R_PPSSIZE  The pointer protection stack size  */
 /*	R_NSIZE	   The number of cons cells	 */
 /*	R_VSIZE	   The vector heap size in bytes */
-/*  These values are defaults and can be overriden in config.h	
+/*  These values are defaults and can be overridden in config.h
     The maxima and minima are in ../unix/sys-common.c */
 
 #ifndef R_PPSSIZE
@@ -62,10 +62,6 @@
 #endif
 
 #include <math.h>
-#ifdef Macintosh
-#define PosixArith
-#define QUICKDRAW_GRAPHICS
-#endif
 
 /* all these are in Rinternals.h
 #include <errno.h>
@@ -79,19 +75,6 @@
 #include <ctype.h>
 */
 
-/* Formerly in Arith.h */
-#ifdef IEEE_754
-# define MATH_CHECK(call)	(call)
-#else
-# ifdef __MAIN__
-    double R_tmp;
-# else
-    extern double R_tmp;
-# endif
-#  define MATH_CHECK(call)	(errno=0,R_tmp=call,(errno==0)?R_tmp:R_NaN)
-#endif
-
-
 /* Getting the working directory */
 #if defined(HAVE_GETCWD)
 #define R_GETCWD(x, y) getcwd(x, y)
@@ -171,6 +154,9 @@ typedef struct {
 #define PRVALUE(x)	((x)->u.promsxp.value)
 #define PRSEEN(x)	((x)->sxpinfo.gp)
 
+/* Hashing Macros */
+#define HASHASH(x)      ((x)->sxpinfo.gp)
+#define HASHVALUE(x)    ((x)->u.vecsxp.truelength)
 
 /* Vector Heap Structure */
 typedef struct {
@@ -300,6 +286,14 @@ FUNTAB	R_FunTab[];	    /* Built in functions */
 #define INI_as(v)
 #endif
 
+/* Formerly in Arith.h */
+#ifdef IEEE_754
+# define MATH_CHECK(call)	(call)
+#else
+  extern double R_tmp;
+# define MATH_CHECK(call)	(errno=0,R_tmp=call,(errno==0)?R_tmp:R_NaN)
+#endif
+
 /* extern int	errno; already have errno.h ! */
 extern int	gc_inhibit_torture INI_as(1);
 
@@ -335,6 +329,9 @@ extern int	R_EvalDepth	INI_as(0);	/* Evaluation recursion depth */
 extern int	R_EvalCount	INI_as(0);	/* Evaluation count */
 extern int	R_BrowseLevel	INI_as(0);	/* how deep the browser is */
 
+extern int	R_Expressions	INI_as(500);	/* options(expressions) */
+extern int	R_KeepSource	INI_as(0);	/* options(keep.source) */
+
 /* File Input/Output */
 extern int	R_Interactive	INI_as(1);	/* Non-zero during interactive use */
 extern int	R_Quiet		INI_as(0);	/* Be as quiet as possible */
@@ -367,7 +364,11 @@ extern int	R_HistorySize;	/* Size of the history file */
 /* Warnings/Errors */
 extern int	R_CollectWarnings INI_as(0);	/* the number of warnings */
 extern SEXP	R_Warnings;	    /* the warnings and their calls */
-extern int	R_ShowErrorMessages INI_as(1);  /* show error messages? */
+extern int	R_ShowErrorMessages INI_as(1);	/* show error messages? */
+
+/* GUI type */
+
+extern char*	R_GUIType	INI_as("unknown");
 
 #ifdef __MAIN__
 #undef extern
@@ -438,6 +439,8 @@ extern int	R_ShowErrorMessages INI_as(1);  /* show error messages? */
 #define NewEnvironment		Rf_NewEnvironment
 #define OneIndex		Rf_OneIndex
 #define onintr			Rf_onintr
+#define onsigusr1               Rf_onsigusr1
+#define onsigusr2               Rf_onsigusr2
 #define parse			Rf_parse
 #define PrintGreeting		Rf_PrintGreeting
 #define PrintVersion		Rf_PrintVersion
@@ -496,9 +499,6 @@ void checkArity(SEXP, SEXP);
 void CheckFormals(SEXP);
 SEXP classgets(SEXP, SEXP);
 void CleanEd(void);
-#ifdef Macintosh
-	void CleanUpMemory( void );
-#endif
 void compactPhase(void);
 void DataFrameClass(SEXP);
 SEXP ddfindVar(SEXP, SEXP);
@@ -555,6 +555,8 @@ SEXP mkSYMSXP(SEXP, SEXP);
 SEXP mkTrue(void);
 SEXP NewEnvironment(SEXP, SEXP, SEXP);
 void onintr();
+void onsigusr1();
+void onsigusr2();
 int OneIndex(SEXP, SEXP, int, int, SEXP*);
 SEXP parse(FILE*, int);
 void PrintGreeting(void);
@@ -563,6 +565,7 @@ void PrintWarnings(void);
 SEXP promiseArgs(SEXP, SEXP);
 void RemoveClass(SEXP, char *);
 SEXP R_LoadFromFile(FILE*, int);
+extern int R_Newhashpjw(char*);
 FILE* R_OpenLibraryFile(char *);
 void R_PreserveObject(SEXP);
 void R_ReleaseObject(SEXP);
@@ -600,6 +603,20 @@ int yyparse(void);
 void yyprompt(char *format, ...);
 int yywrap(void);
 
+/* Macros for suspending interrupts */
+#ifdef HAVE_POSIX_SETJMP
+#define BEGIN_SUSPEND_INTERRUPTS do { \
+    sigset_t mask, omask; \
+    sigemptyset(&mask); \
+    sigaddset(&mask,SIGINT); \
+    sigprocmask(SIG_BLOCK, &mask, &omask);
+#define END_SUSPEND_INTERRUPTS sigprocmask(SIG_SETMASK, &omask, &mask); \
+    } while(0)
+#else
+#define BEGIN_SUSPEND_INTERRUPTS do {
+#define END_SUSPEND_INTERRUPTS } while (0)
+#endif
+
 #endif
 /*
  *- Local Variables:
