@@ -45,7 +45,8 @@ lm <- function (formula, data = list(), subset, weights, na.action,
 	    if (is.matrix(y))
 		c("mlm.null", "lm.null", "mlm", "lm")
 	    else c("lm.null", "lm")
-    } else {
+    }
+    else {
 	x <- model.matrix(mt, mf, contrasts)
 	z <- if(is.null(w)) lm.fit(x, y, offset=offset)
 	else lm.wfit(x, y, w, offset=offset)
@@ -348,20 +349,21 @@ print.summary.lm <-
     invisible(x)
 }
 
-## Commented by KH on 1998/07/10
-## update.default() should be more general now ...
-## update.lm <- function(lm.obj, formula, data, weights, subset, na.action)
-## .....
+## KH on 1998/07/10: update.default() is now used ...
 
-residuals.lm <- function(object,
-                         type = c("working", "pearson", "deviance"), ...)
+residuals.lm <-
+    function(object,
+             type = c("working","response", "deviance","pearson", "partial"),
+             ...)
 {
     type <- match.arg(type)
     r <- .Alias(object$residuals)
     switch(type,
-           working = r,
+           working =, response = r,
            deviance=,
-           pearson =if(is.null(object$weights)) r else r * sqrt(object$weights))
+           pearson =if(is.null(object$weights)) r else r * sqrt(object$weights),
+	   partial = r + predict(object,type="terms")
+           )
 }
 fitted.lm <- function(object, ...) object$fitted.values
 coef.lm <- function(object, ...) object$coefficients
@@ -482,31 +484,31 @@ predict.lm <- function(object, newdata,
                        level=.95,  type=c("response","terms"),
                        terms=NULL, ...)
 {
-    attrassign<-function (object, ...) UseMethod("attrassign")
-    attrassign.lm<-function (lmobj)  attrassign(model.matrix(lmobj), terms(lmobj))
-    attrassign.default<-function (mmat, tt) {
-      if (!inherits(tt, "terms"))
-        stop("need terms object")
-      aa <- attr(mmat, "assign")
-      if (is.null(aa))
-        stop("argument is not really a model matrix")
-      ll <- attr(tt, "term.labels")
-      if (attr(tt, "intercept") > 0)
-        ll <- c("(Intercept)", ll)
-      aaa <- factor(aa, labels = ll)
-      split(order(aa), aaa)
+    attrassign <- function (object, ...) UseMethod("attrassign")
+    attrassign.lm <- function (lmobj)
+        attrassign(model.matrix(lmobj), terms(lmobj))
+    attrassign.default <- function (mmat, tt)
+    {
+        if (!inherits(tt, "terms"))
+            stop("need terms object")
+        aa <- attr(mmat, "assign")
+        if (is.null(aa))
+            stop("argument is not really a model matrix")
+        ll <- attr(tt, "term.labels")
+        if (attr(tt, "intercept") > 0)
+            ll <- c("(Intercept)", ll)
+        aaa <- factor(aa, labels = ll)
+        split(order(aa), aaa)
     }
+    tt <- terms(object)
     if(missing(newdata)) {
         X <- model.matrix(object)
         offset <- object$offset
-        tt<-terms(object)
-    }
-    else {
-        tt <- terms(object)
+    } else {
 	X <- model.matrix(delete.response(tt), newdata,
 			  contrasts = object$contrasts, xlev = object$xlevels)
 	offset <- if (!is.null(off.num<-attr(tt,"offset")))
-	    eval(attr(tt,"variables")[[off.num+1]], newdata)
+	    eval(attr(tt,"variables")[[off.num + 1]], newdata)
 	else if (!is.null(object$offset))
 	    eval(object$call$offset, newdata)
     }
@@ -518,7 +520,7 @@ predict.lm <- function(object, newdata,
     predictor <- drop(X[, piv, drop = FALSE] %*% est)
     if ( !is.null(offset) ) predictor <- predictor + offset
     interval <- match.arg(interval)
-    type<-match.arg(type)
+    type <- match.arg(type)
     if(se.fit || interval != "none") {
 	if (is.null(scale)) {
 	    r <- resid(object)
@@ -530,67 +532,91 @@ predict.lm <- function(object, newdata,
 	} else {
 	    res.var <- scale^2
 	}
-	R <- chol2inv(object$qr$qr[p1, p1, drop = FALSE])
-	vcov <- res.var * R
+
         if (type != "terms"){
-            ip <- real(NROW(X))
-	    for (i in (1:NROW(X))) {
-	       xi <- X[i, piv]
-	       ip[i] <- xi %*% vcov %*% xi
+            if(missing(newdata)) {
+                ip <- drop(qr.Q(object$qr)^2 %*%
+                           rep(res.var, length(object$qr$pivot)))
+            } else {
+                vcov <- res.var * chol2inv(object$qr$qr[p1, p1, drop = FALSE])
+                X1 <- X[, piv, drop = FALSE]
+                ip <- sapply(split(X1, row(X1)),
+                             function(xi) xi %*% vcov %*% xi)
             }
 	}
     }
-    if (type=="terms"){
-      asgn <- attrassign(object)
-      beta<-coef(object)
-      hasintercept<-attr(tt,"intercept")>0
-      if (hasintercept)
-        asgn$"(Intercept)"<-NULL
-      nterms<-length(asgn)
-      predictor<-matrix(ncol=nterms,nrow=NROW(X))
-      dimnames(predictor)<-list(rownames(X),names(asgn))
-      if (se.fit){
-        ip<-matrix(ncol=nterms,nrow=NROW(X))
-        dimnames(ip)<-list(rownames(X),names(asgn))
-      }
-      for (i in seq(1,nterms,length=nterms)){
+    if (type == "terms"){
+        asgn <- attrassign(object)
+        beta <- coef(object)
+        hasintercept <- attr(tt,"intercept")>0
         if (hasintercept)
-          i0<-1
-        else
-          i0<-NULL
-        ii<-piv[asgn[[i]]]
-        predictor[,i]<-X[,ii,drop=F]%*%(beta[ii])
-        X[,ii]<-X[,ii]-mean(X[,ii])
-        if (se.fit){
-          vci<-R[ii,ii]*res.var
-          for(j in (1:NROW(X))){
-            xi<-X[j,ii,drop=F]*(beta[ii])
-            ip[j,i]<-sum(xi%*% vci %*%t(xi))
-          }
+            asgn$"(Intercept)" <- NULL
+        nterms <- length(asgn)
+        predictor <- matrix(ncol=nterms,nrow=NROW(X))
+        dimnames(predictor) <- list(rownames(X),names(asgn))
+
+        if (se.fit||interval!="none"){
+            ip <- matrix(ncol=nterms, nrow=NROW(X))
+            dimnames(ip) <- list(rownames(X), names(asgn))
+            Q <- qr.Q(object$qr)
+            #Rinv <- qr.solve(qr.R(object$qr))
+            Rinv <- chol2inv(qr.R(object$qr))
+            p2 <- dim(Rinv)[2]
         }
-      }
-      if (!is.null(terms)){
-        predictor<-predictor[,terms,drop=F]
-        if (se.fit)
-          ip<-ip[,terms,drop=F]
-      }
-      attr(predictor, 'constant') <- if (hasintercept) coef(object)["(Intercept)"] else 0
+        if(hasintercept){
+            avii <- apply(X, 2, mean)
+            X <- sweep(X, 2, avii)
+        }
+        for (i in seq(1, nterms,length=nterms)){
+            ii <- piv[asgn[[i]]]
+            predictor[,i]  <- X[, ii, drop=FALSE] %*% (beta[ii])
+
+            if (se.fit || interval != "none"){
+                ii2 <- asgn[[i]]
+                vci <- Rinv[ii2, ii2]*res.var
+                for(j in (1:NROW(X))){
+                    xi <- X[j, ii, drop=FALSE]
+                    ip[j,i] <- sum(xi%*% vci %*%t(xi))
+                }
+            }
+        }
+
+        if (!is.null(terms)){
+            predictor <- predictor[, terms, drop=FALSE]
+            if (se.fit)
+                ip <- ip[, terms, drop=FALSE]
+        }
+        attr(predictor, 'constant') <- if (hasintercept)
+            coef(object)["(Intercept)"] else 0
     }
+
     if(interval != "none") {
 	tfrac <- qt((1 - level)/2,df)
 	w <- tfrac * switch(interval,
-			    confidence=sqrt(ip),
-			    prediction=sqrt(ip+res.var)
+			    confidence = sqrt(ip),
+			    prediction = sqrt(ip + res.var)
 			    )
-	predictor <- cbind(predictor, predictor + w %o% c(1,-1))
-	colnames(predictor) <- c("fit","lwr","upr")
+        if(type != "terms"){
+            predictor <- cbind(predictor, predictor + w %o% c(1,-1))
+            colnames(predictor) <- c("fit", "lwr", "upr")
+        }
+        else {
+            lwr <- predictor + w
+            upr <- predictor - w
+        }
     }
-    if(se.fit)
-	list(fit = predictor, se.fit = sqrt(ip),
+    if(type == "terms" && interval != "none")
+	list(fit = predictor, se.fit = sqrt(ip), lwr=lwr, upr=upr,
+	     df = df, residual.scale = sqrt(res.var))
+    else if (se.fit)
+        list(fit = predictor, se.fit = sqrt(ip),
 	     df = df, residual.scale = sqrt(res.var))
     else predictor
 }
 
+
+
+
 effects.lm <- function(object, set.sign = FALSE)
 {
     eff <- object$effects
