@@ -5,9 +5,12 @@ X11 <- function(display="", width=7, height=7, pointsize=12,
 
 x11 <- .Alias(X11)
 
-# Should also rename internal name from "Gnome" to "gnome" ??
 gnome <- function(display="", width=7, height=7, pointsize=12)
-    .Internal(Gnome(display, width, height, pointsize))
+    .Internal(gnome(display, width, height, pointsize))
 
 ## no Gnome <- .Alias(gnome)
 GNOME <- .Alias(gnome)
+
+gtk <- function(display="", width=7, height=7, pointsize=12)
+    .Internal(GTK(display, width, height, pointsize))
+GTK <- .Alias(gtk)
