@@ -346,7 +346,7 @@ SEXP do_symbol(SEXP call, SEXP op, SEXP args, SEXP env)
     char buf[128], *p, *q;
     checkArity(op, args);
     if(!isValidString(CAR(args)))
-	errorcall(call, "invalid argument");
+	errorcall(call, R_MSG_IA);
     p = CHAR(STRING(CAR(args))[0]);
     q = buf;
     while ((*q = *p) != '\0') {
@@ -370,7 +370,7 @@ SEXP do_isloaded(SEXP call, SEXP op, SEXP args, SEXP env)
     int val;
     checkArity(op, args);
     if(!isValidString(CAR(args)))
-	errorcall(call, "invalid argument");
+	errorcall(call, R_MSG_IA);
     sym = CHAR(STRING(CAR(args))[0]);
     val = 1;
     if (!(fun = R_FindSymbol(sym, "")))
@@ -1818,6 +1818,8 @@ void call_R(char *func, long nargs, void **arguments, char **modes,
     SEXP call, pcall, s;
     SEXPTYPE type;
     int i, j, n;
+    char **cptr;
+
     if (!isFunction((SEXP)func))
 	error("invalid function in call_R");
     if (nargs < 0)
@@ -1851,21 +1853,23 @@ void call_R(char *func, long nargs, void **arguments, char **modes,
 	case STRSXP:
 	    n = lengths[i];
 	    CAR(pcall) = allocVector(STRSXP, n);
-	    for (j = 0 ; j < n ; j++) {
-		s = allocSExp(CHARSXP);
-		CHAR(s) = (char*)(arguments[i]);
-		LENGTH(s) = strlen(CHAR(s));
-		STRING(CAR(pcall))[i] = s;
-	    }
+	    cptr = (char**)arguments[i];
+ 	    for (j = 0 ; j < n ; j++) {
+		STRING(CAR(pcall))[j] = mkChar(cptr[j]);
+ 	    }
+	    LENGTH(CAR(pcall)) = n;
 	    break;
 	    /* FIXME : This copy is unnecessary! */
+	    /* FIXME : This is obviously incorrect so disable
 	case VECSXP:
 	    n = lengths[i];
 	    CAR(pcall) = allocVector(VECSXP, n);
 	    for (j = 0 ; j < n ; j++) {
 		VECTOR(s)[i] = (SEXP)(arguments[i]);
 	    }
-	    break;
+	    break; */
+	default:
+	    error("Mode `%s' is not supported in call_R", modes[i]);
 	}
 	if(names && names[i])
 	    TAG(pcall) = install(names[i]);
