@@ -83,7 +83,7 @@
 
 */
 
-static int newhashpjw(char *s)
+extern int R_Newhashpjw(char *s)
 {
     char *p;
     unsigned h = 0, g;
@@ -155,17 +155,23 @@ SEXP R_HashGet(int hashcode, SEXP symbol, SEXP table)
 {
     SEXP chain;
 
+#if 0
+/* Removed by pd -- never seen the "3rd arg" stuff and when the table
+   is a VECSXP it can't be NULL...*/
+
     /* Do type checking */
     if (TYPEOF(table) != VECSXP){
 	printf("3rd arg (table) not of type VECSXP, from R_HashGet\n");
     }
+
     if (isNull(table)) {
 	error("Table is null, from R_HashGet");
     }
+#endif
     /* Grab the chain from the hashtable */
     chain = VECTOR(table)[hashcode];
     /* Retrieve the value from the chain */
-    for (; !isNull(chain); chain = CDR(chain)) {
+    for (; chain != R_NilValue ; chain = CDR(chain)) {
 	if (TAG(chain) == symbol) {
 	    return CAR(chain);
 	}
@@ -300,7 +306,7 @@ SEXP R_HashResize(SEXP table)
     for (counter = 0; counter < length(table); counter++) {
 	chain = VECTOR(table)[counter];
 	while (!isNull(chain)) {
-	    new_hashcode = newhashpjw(CHAR(PRINTNAME(TAG(chain)))) %
+	    new_hashcode = R_Newhashpjw(CHAR(PRINTNAME(TAG(chain)))) %
 		HASHSIZE(new_table);
 	    new_chain = VECTOR(new_table)[new_hashcode];
 	    /* If using a primary slot then increase HASHPRI */
@@ -378,7 +384,12 @@ SEXP R_HashFrame(SEXP rho)
     table = HASHTAB(rho);
     frame = FRAME(rho);
     while (!isNull(frame)) {
-	hashcode = newhashpjw(CHAR(PRINTNAME(TAG(frame)))) % HASHSIZE(table);
+	if( !HASHASH(PRINTNAME(TAG(frame))) ) {
+	    HASHVALUE(PRINTNAME(TAG(frame))) =
+		R_Newhashpjw(CHAR(PRINTNAME(TAG(frame))));
+	    HASHASH(PRINTNAME(TAG(frame))) = 1;
+	}
+	hashcode = HASHVALUE(PRINTNAME(TAG(frame))) % HASHSIZE(table);
 	chain = VECTOR(table)[hashcode];
 	/* If using a primary slot then increase HASHPRI */
 	if (isNull(chain)) HASHPRI(table)++;
@@ -477,6 +488,7 @@ void InitGlobalEnv()
 void unbindVar(SEXP symbol, SEXP rho)
 {
     int hashcode;
+    SEXP c;
     if (HASHTAB(rho) == R_NilValue) {
 	SEXP *v = &(FRAME(rho));
 	while (*v != R_NilValue) {
@@ -489,8 +501,12 @@ void unbindVar(SEXP symbol, SEXP rho)
 	}
     }
     else {
-	hashcode = newhashpjw(CHAR(PRINTNAME(symbol))) %
-	    HASHSIZE(HASHTAB(rho));
+	c = PRINTNAME(symbol);
+	if( !HASHASH(c) ) {
+	    HASHVALUE(c) = R_Newhashpjw(CHAR(c));
+	    HASHASH(c) = 1;
+	}
+	hashcode = HASHVALUE(c) % HASHSIZE(HASHTAB(rho));
 	R_HashDelete(hashcode, symbol, HASHTAB(rho));
     }
 }
@@ -510,7 +526,7 @@ void unbindVar(SEXP symbol, SEXP rho)
 SEXP findVarLocInFrame(SEXP rho, SEXP symbol)
 {
     int hashcode;
-    SEXP frame;
+    SEXP frame, c;
     if (HASHTAB(rho) == R_NilValue) {
 	frame = FRAME(rho);
 	while (frame != R_NilValue && TAG(frame) != symbol)
@@ -518,8 +534,12 @@ SEXP findVarLocInFrame(SEXP rho, SEXP symbol)
 	return frame;
     }
     else {
-	hashcode = newhashpjw(CHAR(PRINTNAME(symbol))) %
-	    HASHSIZE(HASHTAB(rho));
+	c = PRINTNAME(symbol);
+	if( !HASHASH(c) ) {
+	    HASHVALUE(c) = R_Newhashpjw(CHAR(c));
+	    HASHASH(c) = 1;
+	}
+	hashcode = HASHVALUE(c) % HASHSIZE(HASHTAB(rho));
 	/* Will return 'R_NilValue' if not found */
 	return(R_HashGetLoc(hashcode, symbol, HASHTAB(rho)));
     }
@@ -542,7 +562,7 @@ SEXP findVarLocInFrame(SEXP rho, SEXP symbol)
 SEXP findVarInFrame(SEXP rho, SEXP symbol)
 {
     int hashcode;
-    SEXP frame;
+    SEXP frame, c;
     if (HASHTAB(rho) == R_NilValue) {
 	frame = FRAME(rho);
 	while (frame != R_NilValue) {
@@ -552,8 +572,12 @@ SEXP findVarInFrame(SEXP rho, SEXP symbol)
 	}
     }
     else {
-	hashcode = newhashpjw(CHAR(PRINTNAME(symbol))) %
-	    HASHSIZE(HASHTAB(rho));
+	c = PRINTNAME(symbol);
+	if( !HASHASH(c) ) {
+	    HASHVALUE(c) = R_Newhashpjw(CHAR(c));
+	    HASHASH(c) = 1;
+	}
+	hashcode = HASHVALUE(c) % HASHSIZE(HASHTAB(rho));
 	/* Will return 'R_UnboundValue' if not found */
 	return(R_HashGet(hashcode, symbol, HASHTAB(rho)));
     }
@@ -759,7 +783,7 @@ SEXP findFun(SEXP symbol, SEXP rho)
 void defineVar(SEXP symbol, SEXP value, SEXP rho)
 {
     int hashcode;
-    SEXP frame;
+    SEXP frame, c;
     R_DirtyImage = 1;
     if (HASHTAB(rho) == R_NilValue) {
 	if (rho != R_NilValue) {
@@ -779,9 +803,12 @@ void defineVar(SEXP symbol, SEXP value, SEXP rho)
 	SYMVALUE(symbol) = value;
     }
     else {
-	/* Do the hash table thing */
-	hashcode = newhashpjw(CHAR(PRINTNAME(symbol))) %
-	    HASHSIZE(HASHTAB(rho));
+	c = PRINTNAME(symbol);
+	if( !HASHASH(c) ) {
+	    HASHVALUE(c) = R_Newhashpjw(CHAR(c));
+	    HASHASH(c) = 1;
+	}
+	hashcode = HASHVALUE(c) % HASHSIZE(HASHTAB(rho));
 	R_HashSet(hashcode, symbol, HASHTAB(rho), value);
     }
 }
@@ -800,7 +827,7 @@ void defineVar(SEXP symbol, SEXP value, SEXP rho)
 SEXP setVarInFrame(SEXP rho, SEXP symbol, SEXP value)
 {
     int hashcode;
-    SEXP frame;
+    SEXP frame, c;
     if (HASHTAB(rho) == R_NilValue) {
 	frame = FRAME(rho);
 	while (frame != R_NilValue) {
@@ -813,8 +840,12 @@ SEXP setVarInFrame(SEXP rho, SEXP symbol, SEXP value)
     }
     else {
 	/* Do the hash table thing */
-	hashcode = newhashpjw(CHAR(PRINTNAME(symbol))) %
-	    HASHSIZE(HASHTAB(rho));
+	c = PRINTNAME(symbol);
+	if( !HASHASH(c) ) {
+	    HASHVALUE(c) = R_Newhashpjw(CHAR(c));
+	    HASHASH(c) = 1;
+	}
+	hashcode = HASHVALUE(c) % HASHSIZE(HASHTAB(rho));
 	R_HashSet(hashcode, symbol, HASHTAB(rho), value);
     }
     return R_NilValue;
@@ -879,13 +910,17 @@ void gsetVar(SEXP symbol, SEXP value, SEXP rho)
 static SEXP mfindVarInFrame(SEXP rho, SEXP symbol)
 {
     int hashcode;
-    SEXP frame;
+    SEXP frame, c;
     if (HASHTAB(rho) == R_NilValue) {
 	frame = FRAME(rho);
     }
     else {
-	hashcode = newhashpjw(CHAR(PRINTNAME(symbol))) %
-	    HASHSIZE(HASHTAB(rho));
+	c = PRINTNAME(symbol);
+	if( !HASHASH(c) ) {
+	    HASHVALUE(c) = R_Newhashpjw(CHAR(c));
+	    HASHASH(c) = 1;
+	}
+	hashcode = HASHVALUE(c) % HASHSIZE(HASHTAB(rho));
 	frame = VECTOR(HASHTAB(rho))[hashcode];
     }
     while (frame != R_NilValue) {
@@ -994,7 +1029,10 @@ SEXP do_remove(SEXP call, SEXP op, SEXP args, SEXP rho)
     for (i = 0; i < LENGTH(name); i++) {
 	done = 0;
 	tsym = install(CHAR(STRING(name)[i]));
-	hashcode = newhashpjw(CHAR(PRINTNAME(tsym)));
+	if( !HASHASH(PRINTNAME(tsym)) )
+	    hashcode = R_Newhashpjw(CHAR(PRINTNAME(tsym)));
+	else
+	    hashcode = HASHVALUE(PRINTNAME(tsym));
 	tenv = envarg;
 	while (tenv != R_NilValue) {
 	    done = RemoveVariable(tsym, hashcode, tenv);
@@ -1570,20 +1608,20 @@ static SEXP pos2env(int pos, SEXP call)
 {
     SEXP env;
     if (pos == NA_INTEGER || pos < -1 || pos == 0) {
-	errorcall(call, "invalid argument");
+	errorcall(call, R_MSG_IA);
 	env = call;/* just for -Wall */
     }
     else if (pos == -1) {
 	env = R_GlobalContext->sysparent;
 	if (R_GlobalEnv != R_NilValue && env == R_NilValue)
-	    errorcall(call, "invalid argument");
+	    errorcall(call, R_MSG_IA);
     }
     else {
 	for (env = R_GlobalEnv; env != R_NilValue && pos > 1;
 	     env = ENCLOS(env))
 	    pos--;
 	if (pos != 1)
-	    error("invalid argument");
+	    error(R_MSG_IA);
     }
     return env;
 }
