@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  *
  *  SYNOPSIS
  *
@@ -33,7 +33,7 @@
  *    by W. Fullerton of Los Alamos Scientific Laboratory.
  */
 
-#include "Mathlib.h"
+#include "nmath.h"
 
 /* FIXME: We need an ifdef'ed version of this which gives  */
 /* the exact values when we are using IEEE 754 arithmetic. */
